/**
 * Copyright (c) 2011 Colony Project
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

/**
 * @file encoders.cpp
 * @brief Encoders
 *
 * Implementation of functions for encoder use.
 *
 * @author Colony Project, CMU Robotics Club
 * @author Alex Zirbel
 **/

#include <ros/ros.h>

#include "encoders.h"

/**
 * @defgroup encoders Encoders
 * @brief Functions for using the encoders
 *
 * @{
 */

/* Encoder state variables */
/** @todo Fix types: static */
/** @todo Fix integer overflow possibility */

/* The current position of each encoder, measured in encoder ticks */
int encoder_fl_index;
int encoder_fr_index;
int encoder_bl_index;
int encoder_br_index;

/* The current distance each encoder has traveled in ticks from last reset */
int encoder_fl_distance;
int encoder_fr_distance;
int encoder_bl_distance;
int encoder_br_distance;

/**
 * @brief Outputs current encoder data
 *
 * Serves the service query_encoders by responding to service requests with the
 * encoder values.
 * @param req The request. The units that the response should be in.
 * @param res The response. The fields will be filled with values.
 */
bool handle_encoders_query(encoders::query_encoders::Request  &req,
                           encoders::query_encoders::Response &res)
{
    /* Put index, velocity, and distance data in message */
    res.fl_distance = encoder_fl_distance;
    res.fr_distance = encoder_fr_distance;
    res.bl_distance = encoder_bl_distance;
    res.br_distance = encoder_br_distance;

    ROS_DEBUG("Encoder values queried [Unimplemented]");
    return true;
}

/**
 * @brief Encoders driver. This is a ROS node that controls encoders.
 *
 * This is the main function for the encoders node. It is run when the node
 * starts and initializes the encoders. It then publishes to the
 * encoder_state topic and advertises the query_encoders service.
 */
int main(int argc, char **argv)
{
    /* Initialize in ROS the encoders driver node */
    ros::init(argc, argv, "encoders_driver");

    /* Advertise that this serves the query_encoders service */
    ros::NodeHandle n;
    ros::ServiceServer service =
        n.advertiseService("query_encoders",
                           handle_encoders_query);

    /* Initialize hardware for motors */
    // Hardware init functions here

    ROS_INFO("Ready to set encoders.");

    // Acutally measure encoders and update values here

    ros::spin();

    return 0;
}

/** @} */
