/**
 * Copyright (c) 2011 Colony Project
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

/**
 * @file motors.h
 * @brief Contains motor declarations and functions.
 * 
 * Contains functions and definitions for the use of
 * motors.
 *
 * @author Colony Project, CMU Robotics Club
 * @author Benjamin Wasserman
 * @author Alex Zirbel
 * @author Tom Mullins
 **/

#ifndef _MOTORS_H_
#define _MOTORS_H_

#include <fstream>
#include <motors/set_motors.h>
#include <motors/query_motors.h>

#define QUEUE_SIZE 10

/** @brief Initialize the motors module and driver. **/
int main(int argc, char **argv);

/** @brief Responds to topic to set motor speeds and configs. **/
void motors_set(const motors::set_motors::ConstPtr& msg);

/** @brief Responds to service to query motor speeds. **/
bool motors_query(motors::query_motors::Request &req,
                  motors::query_motors::Response &res);

class Motor
{
    public:
        Motor(int pwm_gpt, int in1_gpio, int in2_gpio);
        ~Motor();
        int get_speed();
        void set_speed(int speed);
    private:
        int speed; /**< The current speed of the motor */
        std::fstream fpwm; /**< The device file controlling PWM */
        std::fstream fin1; /**< The device file controlling IN1 */
        std::fstream fin2; /**< The device file controlling IN2 */
};

#endif
